
local addon = KiwiPlates
		
local UnitAura = UnitAura
local UnitIsUnit = UnitIsUnit
local UnitReaction = UnitReaction
local CooldownFrame_Set = CooldownFrame_Set
local C_GetNamePlateForUnit = C_NamePlate.GetNamePlateForUnit

addon.defaults.auras = {}

local buffsDisplayStealable
local buffsGlobalList  

local emptyTable = {}
local emptyFunc  = function() end

local function CreateBuffFrame(UnitFrame)
	UnitFrame.BuffFrame:HookScript('OnShow',function(self) self:Hide() end )
	local buffFrame = CreateFrame( 'Frame', nil, UnitFrame )
	buffFrame.buffList = {}
	UnitFrame.__buffFrame = buffFrame
	buffFrame:Show()
	return buffFrame
end

local function UNIT_AURA(self, unit)

	local plateFrame = C_GetNamePlateForUnit(unit)
	if not plateFrame then return end
	local UnitFrame = plateFrame.UnitFrame
	if not UnitFrame then return end
	local db         = UnitFrame.__skin
	local buffFrame  = UnitFrame.__buffFrame or CreateBuffFrame(UnitFrame)
	local buffList   = buffFrame.buffList
	local buffIndex  = 1
	local filter		
	
	local function CreateAura(name, texture, count, duration, expiration, isBuf)
		local buff = buffList[buffIndex]
		if not buff then
			local w, h, s = db.buffsWidth or 20, db.buffsHeight or 14, db.buffsSpacing or 4
			buff = CreateFrame("Frame", nil, buffFrame, "NameplateBuffButtonTemplate")
			buff:SetScript('OnUpdate', nil)
			buff.layoutIndex = buffIndex
			buff:SetSize( w, h )
			buff:SetPoint( 'BOTTOMLEFT', buffFrame, 'BOTTOMLEFT', (buffIndex-1) * (w+s), 0 )
			buff.Icon:SetSize( w-2, h-2 )
			buff.Icon:SetTexCoord( 0.1, 0.9, 0.1, 0.9 )
			buff.Cooldown:SetSize(w-2,h-2)
			buff:SetMouseClickEnabled(false)
			buff.Border:SetVertexColor(0,1,0,1)
			buffList[buffIndex] = buff
		end
		buff:SetID(buffIndex)
		buff.name = name
		buff.Icon:SetTexture(texture)
		if isBuf then
			buff.Border:SetTexture(1,1,1)
		else
			buff.Border:SetTexture(0,0,0)
		end
		if count > 1 then
			buff.CountFrame.Count:SetText(count)
			buff.CountFrame.Count:Show()
		else
			buff.CountFrame.Count:Hide()
		end
		CooldownFrame_Set(buff.Cooldown, expiration-duration, duration, duration>0, true)
		buff:Show()
		buffIndex = buffIndex + 1	
	end
	
	if UnitIsUnit("player", unit) then
		filter = "HELPFUL|INCLUDE_NAME_PLATE_ONLY"
	else
		local reaction = UnitReaction("player", unit) or 5
		if reaction<5 then
			if buffsDisplayStealable then
				for i = 1, 8 do
					local name, _, texture, count, _, duration, expiration, _, isSteal = UnitAura(unit, i, "HELPFUL")
					if not name then break end
					if isSteal then
						CreateAura(name, texture,count, duration, expiration, true)
					end
				end
			end	
			filter = "HARMFUL|PLAYER|INCLUDE_NAME_PLATE_ONLY"
		else
			filter = "NONE"
		end
	end
	buffFrame.unit   = unit
	buffFrame.filter = filter
	if filter ~= "NONE" then
		local buffsNames  = db.buffsNames or buffsGlobalList
		local isBlackList = db.buffsIsBlackList
		for i = 1, 32 do
			local name, _, texture, count, _, duration, expiration = UnitAura(unit, i, filter)
			if not name then break end
			if isBlackList == not buffsNames[name] then
				CreateAura(name, texture,count, duration, expiration)
			end
		end
	end	
	-- Hide unused buffs and remember visible buffs count
	for i = buffList.__buffMax or 0, buffIndex, -1 do
		buffList[i]:Hide()
	end
	buffList.__buffMax = buffIndex - 1
	
	UnitFrame.BuffFrame:Hide()
end

local function LayoutAuras(self, UnitFrame, db)
	local buffFrame = UnitFrame.__buffFrame or CreateBuffFrame(UnitFrame)
	local buffList  = buffFrame.buffList
	local buffMax   = #buffList
	if buffMax>0 then
		local w, h, s = db.buffsWidth  or 20, db.buffsHeight or 14, db.buffsSpacing or 4
		local x = w+s
		for buffIndex = buffMax, 1, -1 do
			local buff = buffList[buffIndex]
			buff:SetSize( w, h )
			buff:SetPoint( 'BOTTOMLEFT', buffFrame, 'BOTTOMLEFT', (buffIndex-1) * x, 0 )
			buff.Icon:SetSize( w-2, h-2 )
			buff.Icon:SetTexCoord( 0.1, 0.9, 0.1, 0.9 )
			buff.Cooldown:SetSize(w-2,h-2)
		end
	end
	buffFrame:SetSize( 1,1)
	buffFrame:SetPoint("BOTTOMLEFT", UnitFrame.healthBar, "TOPLEFT", db.buffsOffsetX or 0, db.buffsOffsetY or 11)	
	buffFrame:Show()
end

--=============================================================================

local auras = addon:NewModule('auras')

function auras:Update()
	local enabled = addon.db.auras.enabled
	if addon.__userDefinedPlates and enabled~=nil then
		if enabled then -- User defined auras
			addon:RegisterEvent("UNIT_AURA")
			addon.UNIT_AURA       = UNIT_AURA
			addon.LayoutAuras     = LayoutAuras
			buffsGlobalList       = addon.db.auras.buffsNames or emptyTable
			buffsDisplayStealable = addon.db.auras.buffsDisplayStealable		
		else -- Hide auras
			addon:UnregisterEvent("UNIT_AURA")
			addon.UNIT_AURA   = emptyFunc	
			addon.LayoutAuras = emptyFunc
		end
		NamePlateDriverFrame:UnregisterEvent('UNIT_AURA')
	else -- Blizard behavior
		addon:UnregisterEvent("UNIT_AURA")
		addon.UNIT_AURA   = emptyFunc
		addon.LayoutAuras = emptyFunc
		NamePlateDriverFrame:RegisterEvent('UNIT_AURA')
	end
end

--=============================================================================

addon:SetupOptions( 'Skins/Settings', 'Auras', {
	buffsEnabled = {
		type = 'select',
		order = 0.1,
		width = 'normal',
		name = 'Auras Behaviour',
		get = function() 
			local v = addon.db.auras.enabled
			return (v==nil and 1) or (v and 2) or 3
		end,
		set = function(info, v) 
			if v==1 then
				addon.db.auras.enabled= nil
			else
				addon.db.auras.enabled = ( v==2 )
			end
			addon:Update(true)
		end,
		values = { 'Blizzard Default', 'User Defined', 'Hide Auras' },
	},
	buffsSpacing =  {
		type = 'range', order = 0.2, name = 'Auras Spacing', min = 0, softMax = 32, step = 1,
		get = function() return addon:GetSkin().buffsSpacing or 4 end,
		set = function(info,value)
			addon:GetSkin().buffsSpacing = value
			addon:Update(true)
		end,
		disabled = function() return not addon.db.auras.enabled end,		
	},
	buffsOffsetX =  {
		type = 'range', order = 1, name = 'X Adjust', softMin = -64, softMax = 64, step = 1,
		get = function() return addon:GetSkin().buffsOffsetX or 0 end,
		set = function(info,value)
			addon:GetSkin().buffsOffsetX = value~=0 and value or nil
			addon:Update(true)
		end,
		disabled = function() return not addon.db.auras.enabled end,		
	},
	buffsOffsetY =  {
		type = 'range', order = 2, name = 'Y Adjust', softMin = -64, softMax = 64, step = 1,
		get = function() return addon:GetSkin().buffsOffsetY or 11 end,
		set = function(info,value)
			addon:GetSkin().buffsOffsetY = value
			addon:Update(true)
		end,
		disabled = function() return not addon.db.auras.enabled end,		
	},	
	buffsWidth = {
		type = 'range', order = 3, name = 'Aura Width', min = 0, softMax = 64, step = 1,
		get = function() return addon:GetSkin().buffsWidth	or 20 end,
		set = function(info,value)
			addon:GetSkin().buffsWidth = value
			addon:Update(true)					
		end,
		disabled = function() return not addon.db.auras.enabled end,		
	},
	buffsHeight =  {
		type = 'range', order = 4, name = 'Aura Height', min = 1, softMax = 64, step = 1,
		get = function() return addon:GetSkin().buffsHeight or 14 end,
		set = function(info,value)
			addon:GetSkin().buffsHeight = value
			addon:Update(true)
		end,
		disabled = function() return not addon.db.auras.enabled end,		
	},
	buffsBlackList = {
		type = 'select',
		order = 100,
		width = 'normal',
		name = 'Aura List Type',
		desc = 'WhiteList: Specified auras will be displayed.\nBlackList: Specified auras will be hidden.',
		get = function() 
			return addon:GetSkin().buffsIsBlackList and 1 or 2
		end,
		set = function(info, v) 
			addon:GetSkin().buffsIsBlackList = (v==1) or nil
			addon:Update(true)
		end,
		values = { 'BlackList', 'WhiteList' },
		disabled = function() return not addon.db.auras.enabled end,		
	},
	buffsPrivateList = {
		type = "toggle",
		order = 101, width = "normal",
		name = "Use Private List",
		desc = "Use a private aura list specific for this skin instead of the global aura list.",
		get = function() return addon:GetSkin().buffsNamesStr~=nil end,
		set = function (_, value) 
			addon:GetSkin().buffsNamesStr = value and "" or nil
			addon:GetSkin().buffsNames    = value and {} or nil
		end,
		disabled = function() return not addon.db.auras.enabled end,				
		confirm = function(_, value)
			return (not value) and "Current aura list will be erased and the global list will be used. Are you Sure ?" or nil
		end
	},
	buffsNames = {
		type = 'input',
		order = 102,
		name = 'Aura List',
		multiline = 7,
		width = 'full',
		get = function(info)
			return addon:GetSkin().buffsNamesStr or addon.db.auras.buffsNamesStr
		end,
		set = function(info, value)
			local db =  addon:GetSkin().buffsNamesStr and addon:GetSkin() or addon.db.auras
			local names = db.buffsNames or {}
			wipe(names)
			local list  = { strsplit("\n", value) }
			for i=1,#list do
				names[ strtrim(list[i]) ] = true
			end			
			db.buffsNames    = names
			db.buffsNamesStr = value
			addon:Update(true)
		end,
		disabled = function() return not addon.db.auras.enabled end,		
	},
	buffsDisplayStealable = {
		type = "toggle",
		order = 90, width = "full",
		name = "Display Stealable Buffs",
		desc = "Display stealable buffs first highlighted with a green border (only for enemies).",
		get = function() return addon.db.auras.buffsDisplayStealable end,
		set = function (_, value) 
			addon.db.auras.buffsDisplayStealable = value and true or nil
			addon:Update(true)
		end,
		disabled = function() return not addon.db.auras.enabled end,				
	},	
	
} )
